@@ -7,3 +7,5 @@ from .layout import Layout
 from .legenditem import LegendItem
 from .metadata import Metadata
 from .technique import Technique
+from .versions import Versions
+from .helpers import *
